//===============================================================================
// Microsoft Caching Application Block for .NET
// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/dnpag/html/CachingBlock.asp
//
// CacheConfigurationHandler.cs
// The Configuration Section Handler for the "CacheManagerSettings" section of
// the config file. 
//
//===============================================================================
// Copyright (C) 2003 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================


using System;
using System.Collections;
using System.Configuration;
using System.Reflection;
using System.Resources;
using System.Xml;

// Used for handling Exceptions
using Microsoft.ApplicationBlocks.ExceptionManagement;

namespace Microsoft.ApplicationBlocks.Cache
{
	#region CacheConfigurationHandler

	/// <summary>
	///	The Configuration Section Handler for the "CacheManagerSettings"
	///	section of the config file. 
	/// </summary>
	public class CacheConfigurationHandler : IConfigurationSectionHandler
	{
		#region Public constructor

		/// <summary>
		///	Create an instance of this class.
		/// </summary>
		public CacheConfigurationHandler(){}

		#endregion

		#region IConfigurationSectionHandler implementation

		/// <summary>
		///	Implemented by all configuration section handlers to parse the XML 
		///	of the configuration section. The returned object is added to the 
		///	configuration collection and is accessed by GetConfig.
		/// </summary>
		/// <param name="parent">
		///	The configuration settings in a corresponding parent 
		///	configuration section
		///	</param>
		/// <param name="context">
		///	An HttpConfigurationContext when Create is called from the ASP.NET 
		///	configuration system. Otherwise, this parameter is reserved and is
		///	a null reference.  
		/// </param>
		/// <param name="section">
		///	The XmlNode that contains the configuration information from the 
		///	configuration file. Provides direct access to the XML contents
		///	of the configuration section. 
		/// </param>
		/// <returns>
		///	The object of CacheConfigurationSettings
		/// </returns>
		object IConfigurationSectionHandler.Create ( object parent, 
			object context, 
			XmlNode section )		
		{
			try
			{
				if( Object.Equals(section, null))
				{
					throw(new ArgumentNullException(CacheResources. 
						ResourceManager["RES_ExceptionNullSection"]));
				}

				// Read all the information from the config file
				XmlNode storageInfo = section.SelectSingleNode(
					CacheResources.ResourceManager[ "RES_StorageInfo" ] );
				XmlNode scavengingInfo = section.SelectSingleNode(
					CacheResources.ResourceManager[ "RES_ScavengingInfo" ] );
				XmlNode expirationInfo = section.SelectSingleNode(
					CacheResources.ResourceManager[ "RES_ExpirationInfo" ] );
				XmlNode dataProtectionInfo = section.SelectSingleNode(
					CacheResources.ResourceManager[ 
						"RES_DataProtectionInfo" ] );
							
				CacheConfigurationSettings config = 
					new CacheConfigurationSettings( 
						storageInfo,
						scavengingInfo, 
						expirationInfo, 
						dataProtectionInfo );		
				return config;
			}
			catch( Exception genException )
			{
				ExceptionManager.Publish( genException );
				throw;
			}			
		}
		#endregion
	}

	#endregion
}