//===============================================================================
// Microsoft Caching Application Block for .NET
// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/dnpag/html/CachingBlock.asp
//
// CacheConfigurationSettings.cs
// Class that defines the settings within the cache configuration settings in
// the config file.
//
//===============================================================================
// Copyright (C) 2003 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================


using System;
using System.Configuration;
using System.Xml;

// Used for handling Exceptions
using Microsoft.ApplicationBlocks.ExceptionManagement;

namespace Microsoft.ApplicationBlocks.Cache
{
	#region CacheConfigurationSettings

	/// <summary>
	///	Class that defines the settings within the cache configuration settings 
	///	in the config file.
	/// </summary>
	internal class CacheConfigurationSettings
	{
		#region Member Variables

		StorageInfo configStorageSettings;
		ScavengingInfo configScavengingSettings;
		ExpirationInfo configExpirationSettings;
		DataProtectionInfo configDataProtectionSettings;

		#endregion

		#region Constructor

		/// <summary>
		///	Constructor to define the cache configuration settings.		
		/// </summary>
		internal CacheConfigurationSettings ( XmlNode storageInfo,
			XmlNode scavengingInfo, 
			XmlNode expirationInfo, 
			XmlNode dataProtectionInfo )
		{
			try
			{
				if(! Object.Equals(storageInfo, null) )
				{
					configStorageSettings = new StorageInfo( storageInfo );
				}
			
				if(! Object.Equals(scavengingInfo, null) )
				{
					configScavengingSettings = new ScavengingInfo( 
						scavengingInfo );
				}
			
				if(! Object.Equals(expirationInfo, null) )
				{
					configExpirationSettings = new ExpirationInfo( 
						expirationInfo );
				}
				
				if(! Object.Equals(dataProtectionInfo, null) )
				{
					configDataProtectionSettings = new DataProtectionInfo( 
						dataProtectionInfo );
				}
			}
			catch( Exception genException )
			{
				ExceptionManager.Publish( genException );
				throw;
			}
		}

		#endregion

		#region Properties

		/// <summary>
		///	Gets the storage provider settings.
		/// </summary>
		internal StorageInfo StorageInformation
		{ get{ return configStorageSettings; } }		

		/// <summary>
		///	Gets the scavenging algorithm settings.
		/// </summary>
		internal ScavengingInfo ScavengingInformation
		{ get{ return configScavengingSettings; } }		

		/// <summary>
		///	Gets the expiration settings.
		/// </summary>
		internal ExpirationInfo ExpirationInformation
		{ get{ return configExpirationSettings; } }
				
		/// <summary>
		///	Gets the data protection settings.
		/// </summary>
		internal DataProtectionInfo DataProtectionInformation
		{ get{ return configDataProtectionSettings; } }
		
		#endregion
	}
	#endregion	
}