//===============================================================================
// Microsoft Caching Application Block for .NET
// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/dnpag/html/CachingBlock.asp
//
// CacheEventArgs.cs
// This class is derived from the EventArgs class and it wraps the key of the
// cache data to pass it to events.
//
//===============================================================================
// Copyright (C) 2003 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================


using System;

namespace Microsoft.ApplicationBlocks.Cache
{
	/// <summary>
	///	This class is derived from EventArgs and is used to wrap
	///	the cache item key for passing it to event handler.
	/// </summary>
	public class CacheEventArgs : EventArgs
	{
		// Unique value to identify a cache data
		private string key;

		/// <summary>
		///	Property to return the key value.
		/// </summary>
		public string KeyValue
		{
			get
			{
				return key;
			}
		}

		/// <summary>
		/// Constructor to create the CacheEventArgs object.
		/// </summary>
		/// <param name="key">
		/// Key to uniquely identify a value
		/// </param>
		public CacheEventArgs(string key)
		{
			this.key = key;
		}
	}
}
