//===============================================================================
// Microsoft Caching Application Block for .NET
// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/dnpag/html/CachingBlock.asp
//
// CacheItem.cs
// This class is used to get or set the value and metadata of a cache item
//
//===============================================================================
// Copyright (C) 2003 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================


using System;

// Used to handle exceptions
using Microsoft.ApplicationBlocks.ExceptionManagement;

namespace Microsoft.ApplicationBlocks.Cache
{
	/// <summary>
	///	This class represents a cache item. A cache item has a value and 
	///	metadata which can be set or got using this class.
	/// </summary>
	[Serializable]
	public class CacheItem
	{
		#region Member Variables

		string itemKey;
		object itemValue;
		CacheItemPriority itemPriority; 
		ICacheItemExpiration[] itemExpirations;

		#endregion

		#region Constructor

		/// <summary>
		///	Default constructor.
		/// </summary>
		public CacheItem()
		{
			itemPriority = CacheItemPriority.Normal;
		}

		#endregion
		
		#region Properties

		/// <summary>
		///	Gets or Sets the item key.
		/// </summary>
		public string Key
		{
			get{return itemKey;}
			set
			{
				try
				{
					if( Object.Equals(value, null) )
					{
						throw new ArgumentNullException("itemKey", 
							CacheResources.ResourceManager[
							"RES_ExceptionNullItemKey" ] );						
					}
					if( value.Length == 0)
					{
						throw new ArgumentOutOfRangeException(
							"itemKey", CacheResources.ResourceManager[
							"RES_ExceptionEmptyItemKey" ] );
					}
					itemKey = value;
				}
				catch(Exception genException)
				{
					ExceptionManager.Publish(genException);
					throw;
				}
			}
		}
		
		/// <summary>
		///	Gets or Sets the item value.
		/// </summary>
		public object Value
		{
			get{return itemValue;}
			set
			{
				try
				{
					itemValue = value;
				}
				catch(Exception genException)
				{
					ExceptionManager.Publish(genException);
					throw;
				}
			}
		}
		
		/// <summary>
		///	Gets or Sets the item priority.
		/// </summary>
		public CacheItemPriority Priority
		{
			get{return itemPriority;}
			set
			{
				try
				{
					if(!Enum.IsDefined(typeof(CacheItemPriority), value))
					{
						throw new ArgumentOutOfRangeException(
							"CacheItemPriority",
							CacheResources.ResourceManager[
							"RES_ExceptionPriorityEnum"]);
					}
					itemPriority = value;
				}
				catch(Exception genException)
				{
					ExceptionManager.Publish(genException);
					throw;
				}
			}
		}
		
		/// <summary>
		///	Gets or Sets the item expirations.
		/// </summary>
		public ICacheItemExpiration[] Expirations
		{
			get{return itemExpirations;}
			set
			{
				itemExpirations = value;
			}
		}
		#endregion
	}
}