//===============================================================================
// Microsoft Caching Application Block for .NET
// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/dnpag/html/CachingBlock.asp
//
// CacheResources.cs
// This class is used to access the constants defined in the Resources file.
//
//===============================================================================
// Copyright (C) 2003 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================


using System;
using System.Globalization;
using System.Reflection;
using System.Resources;

// Used for handling Exceptions
using Microsoft.ApplicationBlocks.ExceptionManagement;

namespace Microsoft.ApplicationBlocks.Cache
{
	/// <summary>
	///	This class is used to access the constants defined in the
	///	resources file.	
	/// </summary>
	public class CacheResources
	{
		// Member Variables
		static CacheResources resource;
		ResourceManager cachingResourceManager;
		string constantValue;

		#region Properties
		/// <summary>
		///	Indexer to get the value of the key.
		/// </summary>
		public string this [ string key ]
		{
			get
			{
				try
				{
					constantValue = cachingResourceManager.GetString( key, 
						CultureInfo.CurrentCulture );					
				}
				catch(Exception genException)
				{
					ExceptionManager.Publish(genException);
				}
				return constantValue; 
			}
		}
		
		/// <summary>
		///	Gets the resource instance variable.
		/// </summary>
		public static CacheResources ResourceManager
		{
			get
			{
				try
				{
					if(Object.Equals(resource, null))
					{
						resource = new CacheResources();
						return resource;
					}
					else
					{
						return resource;
					}
				}
				catch(Exception genException)
				{
					ExceptionManager.Publish(genException);
				}
				return resource;
			}
		}
		#endregion
		
		#region Instance part
		
		/// <summary>
		///	Constructor to initialize the resource manager 
		///	instance variable.
		/// </summary>
		private CacheResources()
		{
			try
			{
				const string RESOURCE_FILE = ".CacheManagerText";
				cachingResourceManager = 
					new ResourceManager( typeof(CacheManager).Namespace 
					+ RESOURCE_FILE, Assembly.GetExecutingAssembly() );
			}
			catch(Exception genException)
			{
				ExceptionManager.Publish(genException);
			}
		}
		
		#endregion
	}
}
