//===============================================================================
// Microsoft Caching Application Block for .NET
// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/dnpag/html/CachingBlock.asp
//
// DataProtectionInfo.cs
// Class that defines the data protection provider settings within the cache
// configuration settings in the config file.
//
//===============================================================================
// Copyright (C) 2003 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================


using System;
using System.Collections;
using System.Configuration;
using System.Globalization;
using System.Reflection;
using System.Resources;
using System.Xml;

namespace Microsoft.ApplicationBlocks.Cache
{
	#region DataProtection information
	/// <summary>
	///	Class that defines the data protection provider settings within the 
	///	cache configuration settings in the config file.
	/// </summary>
	internal class DataProtectionInfo : ProviderBaseClass
	{
		#region Public constructor

		/// <summary>
		///	Creates an instance of this class by reading the data protection 
		///	provider settings in the config file.
		/// </summary>
		/// <param name="dataProtectionInfo">
		///	The protection information for the data 
		/// </param>
		public DataProtectionInfo( XmlNode dataProtectionInfo )
			: base( dataProtectionInfo )
		{ }

		#endregion
		
		#region Public methods

		/// <summary>
		///	Gets a new IDataProtection instance.
		/// </summary>
		/// <remarks>
		///	IDataProtection dataProtection = CreateInstance();
		/// </remarks>
		/// <returns>
		///	The instance of the IDataProtection interface
		/// </returns>
		public IDataProtection CreateInstance ()
		{
			// Check the provider for the implementation of the desired 
			// interface
			try
			{
				providerType.GetInterfaceMap( typeof(IDataProtection) );
			}
			catch
			{
				throw new Exception( 
					string.Format( 
					CultureInfo.CurrentCulture,
					CacheResources.ResourceManager[ 
					"RES_ExceptionInvalidConfigurationInvalidProviderInterface"
					], 
					this.GetType().Name, 
					providerAssemblyName,
					providerClassName, 
					"IDataProtection" ) );
			}
			
			// Create the provider instance
			try
			{
				object providerInstance = Activator.CreateInstance( 
					providerType );
				IDataProtection dataProtection = 
					(IDataProtection)providerInstance;
				dataProtection.Init( configProviderInfo );
				return dataProtection;
			}
			catch( Exception genException )
			{
				throw new Exception( 
					string.Format( 
					CultureInfo.CurrentCulture, 
					CacheResources.ResourceManager[ 
					"RES_ExceptionInvalidConfigurationProviderInformationCantCreateInstance" ], 
					this.GetType().Name, 
					providerAssemblyName, 
					providerClassName, 
					"IDataProtection" ), 
					genException );
			}
		}

		#endregion
	}

	#endregion
}