//===============================================================================
// Microsoft Caching Application Block for .NET
// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/dnpag/html/CachingBlock.asp
//
// Delegates.cs
// This class has all the delegates required for the caching operations.
//
//===============================================================================
// Copyright (C) 2003 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================


using System;

namespace Microsoft.ApplicationBlocks.Cache
{
	/// <summary>
	///	Represents the method that will handle the dependency change event.
	/// </summary>
	[Serializable]
	public delegate void ItemDependencyChangeEventHandler (
		object sender, 
		CacheEventArgs e); 
	
	/// <summary>
	///	Represents the method that will be invoked when a item is removed. 
	/// </summary>
	[Serializable]
	public delegate void CacheItemRemovedCallback (	string key, 
		CacheItemRemoveCause cause );	

	/// <summary>
	///	Represents the method that will handle the addition of item metadata.
	/// </summary>
	internal delegate void AddDataHandler ( string key,
		ICacheItemExpiration[] expirations,
		CacheItemPriority priority,
		CacheItemRemovedCallback onRemoveCallback );
	
	/// <summary>
	///	Represents the method that will handle the flushing of all the items.
	/// </summary>
	internal delegate void FlushHandler ();

	/// <summary>
	///	Represents the method that will handle the retrieval of the cache item.
	/// </summary>
	internal delegate CacheItem GetHandler ( string key );
	
	/// <summary>
	///	Represents the method that will handle the time when the item was last 
	///	used.
	/// </summary>
	internal delegate void NotifyHandler ( string key );
	
	/// <summary>
	///	Represents the method that will handle the removal of the item 
	///	metadata.
	/// </summary>
	internal delegate void RemoveDataHandler (string key);
	
	/// <summary>
	///	Represents the method that will handle the updation of item metadata.
	/// </summary>
	internal delegate void UpdateHandler ( string key,
	ICacheItemExpiration[] expirations,
	CacheItemPriority priority );

}