//===============================================================================
// Microsoft Caching Application Block for .NET
// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/dnpag/html/CachingBlock.asp
//
// ExpirationInfo.cs
// Class that defines the expiration provider settings within the cache
// configuration settings in the config file.
//
//===============================================================================
// Copyright (C) 2003 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================


using System;
using System.Configuration;
using System.Globalization;
using System.Xml;

// Used to handle exceptions
using Microsoft.ApplicationBlocks.ExceptionManagement;

namespace Microsoft.ApplicationBlocks.Cache
{
	#region Expiration information
	/// <summary>
	///	Class that defines the expiration provider settings within
	///	the cache configuration settings in the config file.
	/// </summary>
	internal class ExpirationInfo
	{
		#region Field members
		int expirationInterval = 0;
		#endregion

		#region Public constructor
		/// <summary>
		///	Constructor to read the expiration information from
		///	the config file.
		/// </summary>
		public ExpirationInfo( XmlNode expirationInfo )
		{
			try
			{
				if(Object.Equals(expirationInfo, null))
				{
					throw(new ArgumentNullException("expirationInfo",
						CacheResources.ResourceManager[
						"RES_ExceptionNullExpirationInfo"]));
				}
				// Check the expiration information
				if( Object.Equals(expirationInfo.Attributes[ 
					CacheResources.ResourceManager ["RES_Interval" ] ], null) )
				{
					throw(new ConfigurationException( 
						CacheResources.ResourceManager[ 
						"RES_ExceptionNullConfigurationExpirationInfo" ] ));
				}

				string intervalString = expirationInfo.Attributes[ 
					CacheResources.ResourceManager ["RES_Interval" ] ].Value;

				// Check the interval
				if( intervalString.Length == 0 )
				{
					throw new ConfigurationException( 
						CacheResources.ResourceManager[ 
						"RES_ExceptionEmptyConfigurationExpirationInfo" ] );
				}

				double  intervalValue;
				if(double.TryParse(intervalString, NumberStyles.Integer,
					null, out intervalValue))
				{
					if(intervalValue > int.MaxValue)
					{
						throw(new ConfigurationException(CacheResources.
							ResourceManager[
							"RES_ExceptionMaxConfigurationExpirationInfo"]));
					}
					expirationInterval = int.Parse(intervalString,
						NumberFormatInfo.CurrentInfo);

					if(expirationInterval < 1)
					{
						throw(new ConfigurationException(CacheResources.
							ResourceManager[
							"RES_ExceptionMinConfigurationExpirationInfo"]));
					}
				}
				else
				{
					throw(new ConfigurationException(CacheResources.
						ResourceManager[
						"RES_ExceptionInvalidConfigurationExpirationInfo"]));
				}
			}
			catch( Exception genException )
			{
				ExceptionManager.Publish(genException);
				throw;
			}
		}

		#endregion

		#region Properties
		/// <summary>
		///	Gets the expiration interval.
		/// </summary>
		public int Interval
		{ get{ return expirationInterval; } }
		#endregion
	}
	#endregion	
}
