//===============================================================================
// Microsoft Caching Application Block for .NET
// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/dnpag/html/CachingBlock.asp
//
// ScavengingInfo.cs
// Class that defines the scavenging algorithm settings within the cache
// configuration settings in the config file.
//
//===============================================================================
// Copyright (C) 2003 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================


using System;
using System.Collections;
using System.Configuration;
using System.Globalization;
using System.Reflection;
using System.Resources;
using System.Xml;

namespace Microsoft.ApplicationBlocks.Cache
{
	#region Scavenging information

	/// <summary>
	///	Class that defines the scavenging algorithm settings within the cache 
	///	configuration settings in the config file.
	/// </summary>
	internal class ScavengingInfo : ProviderBaseClass
	{
		#region Public constructor
		
		/// <summary>
		/// Default constructor.
		/// </summary>
		/// <param name="scavengingInfo">
		/// Xml node which provides the sacvenging information
		/// </param>
		public ScavengingInfo( XmlNode scavengingInfo )
			: base( scavengingInfo )
		{ }

		#endregion

		#region Public methods

		/// <summary>
		///	Gets a new IScavengingAlgorithm instance.
		/// </summary>
		/// <param name="cacheService">
		///	Instance of CacheService
		/// </param>
		/// <param name="cacheStorage">
		///	Instance of ICacheStorage
		/// </param>
		/// <param name="cacheMetadata">
		///	Instance of ICacheMetadata
		/// </param>
		/// <returns>
		///	The instance of the scavenging algorithm
		/// </returns>
		public IScavengingAlgorithm CreateInstance( CacheService cacheService,
			ICacheStorage cacheStorage, 
			ICacheMetadata cacheMetadata )
		{
			try
			{
				// Returns a interface mapping for the specified
				// interface type
				providerType.GetInterfaceMap( typeof(IScavengingAlgorithm) );
			}
			catch
			{
				throw new Exception( 
					string.Format( 
					CultureInfo.CurrentCulture, 
					CacheResources.ResourceManager[ 
					"RES_ExceptionInvalidConfigurationInvalidProviderInterface"
					], 
					this.GetType().Name, 
					providerAssemblyName, 
					providerClassName, 
					"IScavengingAlgorithm" ) );
			}
			
			// Create the provider instance
			try
			{
				object providerInstance = Activator.CreateInstance( 
					providerType );
				IScavengingAlgorithm scavengingAlgorithm = 
					(IScavengingAlgorithm)providerInstance;
				scavengingAlgorithm.Init( cacheService, 
					cacheStorage,
					cacheMetadata, 
					configProviderInfo );
				return scavengingAlgorithm;
			}
			catch( Exception genException )
			{
				throw new Exception( 
					string.Format( 
					CultureInfo.CurrentCulture, 
					CacheResources.ResourceManager[ 
					"RES_ExceptionInvalidConfigurationProviderInformationCantCreateInstance" ], 
					this.GetType().Name,
					providerAssemblyName,
					providerClassName,
					"IScavengingAlgorithm" ), 
					genException );
			}
		}

		#endregion
	}

	#endregion
}