//===============================================================================
// Microsoft Caching Application Block for .NET
// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/dnpag/html/CachingBlock.asp
//
// StorageInfo.cs
// Class that defines the storage provider settings within the cache
// configuration settings in the config file.
//
//===============================================================================
// Copyright (C) 2003 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================


using System;
using System.Globalization;
using System.Xml;

namespace Microsoft.ApplicationBlocks.Cache
{
	#region Storage information

	/// <summary>
	///	Class that defines the storage provider settings within the
	///	cache configuration settings in the config file.
	/// </summary>
	internal class StorageInfo : ProviderBaseClass
	{
		#region Public constructor

		public StorageInfo( XmlNode storageInfo ): base( storageInfo )
		{ }

		#endregion

		#region Public methods

		/// <summary>
		///	Gets a new ICacheStorage instance.
		/// </summary>
		/// <returns>
		///	Returns a interface mapping for the ICacheStorage
		///	interface type
		/// </returns>
		public ICacheStorage CreateInstance()
		{
			try
			{
				// Returns a interface mapping for the specified
				// interface type
				providerType.GetInterfaceMap( typeof(ICacheStorage) );
			}
			catch
			{
				throw new Exception( 
					string.Format( 
					CultureInfo.CurrentCulture, 
					CacheResources.ResourceManager[ 
					"RES_ExceptionInvalidConfigurationInvalidProviderInterface" ], 
					this.GetType().Name, 
					providerAssemblyName, 
					providerClassName, 
					"ICacheStorage" ) );
			}
			
			// Create the provider instance
			try
			{
				object providerInstance = Activator.CreateInstance( 
					providerType );
				ICacheStorage cacheStorage = (ICacheStorage)providerInstance;
				cacheStorage.Init( configProviderInfo );
				return cacheStorage;
			}
			catch( Exception genException )
			{
				throw new Exception( 
					string.Format( 
					CultureInfo.CurrentCulture, 
					CacheResources.ResourceManager[ 
					"RES_ExceptionInvalidConfigurationProviderInformationCantCreateInstance" ], 
					this.GetType().Name, 
					providerAssemblyName, 
					providerClassName, 
					"ICacheStorage" ), 
					genException );
			}
		}

		#endregion
	}

	#endregion
}