//===============================================================================
// Copyright (C) 2003 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================


using System;
using System.Collections;
using System.Configuration;
using System.Diagnostics;
using System.Reflection;
using System.Resources;
using System.Xml;


namespace Microsoft.ApplicationBlocks.Common.Crypto {
    /// <devdoc>
    ///	The Configuration Section Handler for the "cryptographicSettings" section of the config file. 
    /// </devdoc>
    internal sealed class CryptographicConfigurationHandler : IConfigurationSectionHandler {
        #region Constants
        private const String dppXpath = "cryptographicInfo";
        #endregion

        #region Constructors		
        public CryptographicConfigurationHandler(){}
        #endregion

        #region IConfigurationSectionHandler Members
        /// <devdoc>
        ///	Implemented by all configuration section handlers to parse the XML 
        ///	of the configuration section. The returned object is added to the 
        ///	configuration collection and is accessed by GetConfig.
        /// </devdoc>
        Object IConfigurationSectionHandler.Create(object parent, object context, XmlNode section) {
            Hashtable providers = new Hashtable(5);
            if(Object.Equals(section, null)) {
                throw(new ArgumentNullException("section"));
            }
            XmlNodeList nodes = section.SelectNodes(dppXpath);
            Debug.Assert(nodes != null);
            foreach(XmlNode node in nodes) {
                Provider provider = new Provider(node);
                Debug.Assert(provider != null);
				if( providers.Contains(provider.Name.ToLower(System.Globalization.CultureInfo.CurrentCulture)) )
				{
					throw new ConfigurationException(String.Format(ResourceTable.GetString(ResourceTable.DuplicateCryptographicInfoSpecified), provider.Name));	
				}
                providers.Add(provider.Name.ToLower(System.Globalization.CultureInfo.CurrentCulture), provider);
            }
            return providers;
        }
        #endregion
    }
}