//===============================================================================
// Copyright (C) 2003 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================

using System;
using System.Collections.Specialized;
using System.Configuration;
using System.Globalization;
using System.Diagnostics;

namespace Microsoft.ApplicationBlocks.Common.Crypto {
    
    /// <include file='doc\CryptographicProviderManager.uex' path='docs/doc[@for="CryptographicProviderManager"]/*'/>
    /// <devdoc>
    /// Represents a factory class to create ICryptographicProvider instances.
    /// </devdoc>
    public sealed class CryptographicProviderManager : ProviderManagerBase {
        #region Static Attributes
        // the one and only factory
        private static CryptographicProviderManager singleton;
        #endregion       

        #region Constructors
        static CryptographicProviderManager() {
            singleton = new CryptographicProviderManager();
        }

        private CryptographicProviderManager() : base() {
        }
        #endregion

        #region Singleton Instance
        /// <include file='doc\CryptographicProviderManager.uex' path='docs/doc[@for="CryptographicProviderManager.Current"]/*'/>        
        /// <devdoc>
        /// Gets the current factory for creating the providers.
        /// </devdoc>        
        public static CryptographicProviderManager Current {
            get { return singleton; }
        }
        #endregion

        /// <include file='doc\CryptographicProviderManager.uex' path='docs/doc[@for="CryptographicProviderManager.Create"]/*'/>
        /// <devdoc>
        /// Creates the provider for the given name.
        /// </devdoc>        
        public ICryptographicProvider Create(String providerName) {
            IProvider provider =  base.GetProvider("crytpographicSettings", providerName) as ICryptographicProvider;
            try {
                provider.GetType().GetInterfaceMap(typeof(ICryptographicProvider));
            } catch (ArgumentException ex) {
                throw new ConfigurationException( 
                    String.Format( 
                    CultureInfo.CurrentCulture,
                    ResourceTable.GetString(ResourceTable.InvalidConifgInterface), 
                    this.GetType().Name, 
                    provider.GetType(),
                    "ICryptographicProvider"), 
                    ex);
            }
            return provider as ICryptographicProvider;
        }
    }
}
