//===============================================================================
// Copyright (C) 2003 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================


using System;
using System.Collections;
using System.Configuration;
using System.Security.Cryptography;
using System.Xml;

namespace Microsoft.ApplicationBlocks.Common.Crypto {	
    /// <include file='doc\ICryptographicProvider.uex' path='docs/doc[@for="ICryptographicProvider"]/*'/>
    /// <devdoc>
    ///	Represents a data protection schema.
    /// </devdoc>
    public interface ICryptographicProvider : IProvider {
        #region Methods
        /// <include file='doc\ICryptographicProvider.uex' path='docs/doc[@for="ICryptographicProvider.Encrypt"]/*'/>
        /// <devdoc>
        ///	When implemented by a class, encrypts raw bytes.
        /// </devdoc>      
        Byte[] Encrypt(Byte[] plainValue);
		
        /// <include file='doc\ICryptographicProvider.uex' path='docs/doc[@for="ICryptographicProvider.Decrypt"]/*'/>
        /// <devdoc>
        ///	When implemented by a class, decrypts raw bytes.
        /// </devdoc>        
        Byte[] Decrypt(Byte[] cipherValue);
		
        /// <include file='doc\ICryptographicProvider.uex' path='docs/doc[@for="ICryptographicProvider.ComputeHash"]/*'/>
        /// <devdoc>
        ///	Computes a hash for data validation.
        /// </devdoc>        
        Byte[] ComputeHash(Byte[] plainText);
        #endregion
    }
}