//=============================================================================
// Copyright  2003 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//=============================================================================

using System;
using System.Collections;
using System.Collections.Specialized;
using System.Xml;
using System.Xml.XPath;

namespace Microsoft.ApplicationBlocks.Common {
    /// <include file='doc\Provider.uex' path='docs/doc[@for="Provider"]/*'/>
    /// <devdoc>
    /// Represents a provider from the configuration file.
    /// </devdoc>    
    public sealed class Provider {
        #region Attributes
        // the provider name
        private String              providerName;
        // the type for the provider
        private Type                type;
        // the custom properties for the provider
        private XPathNavigator      properties;
        #endregion

        #region Constructors
        /// <include file='doc\Provider.uex' path='docs/doc[@for="Provider.Provider"]/*'/>
        /// <devdoc>
        /// Initialize a new instance of the Provider class with the node from the configuration section.
        /// </devdoc>        
        public Provider(XmlNode node) {
            providerName = node.Attributes.GetNamedItem(ResourceTable.GetString(ResourceTable.Name)).Value.Trim();
            String typeName = node.Attributes.GetNamedItem(ResourceTable.GetString(ResourceTable.Type)).Value.Trim();
            type = Type.GetType(typeName, true, true);            
            properties = node.CreateNavigator();
        }
        #endregion

        #region Properties
        /// <include file='doc\Provider.uex' path='docs/doc[@for="Provider.Name"]/*'/>
        /// <devdoc>
        /// Gets the name of the provider.
        /// </devdoc>        
        public String Name {
            get { return providerName; }
        }

        /// <include file='doc\Provider.uex' path='docs/doc[@for="Provider.Type"]/*'/>
        /// <devdoc>
        /// Gets the Type of the provider.
        /// </devdoc>        
        public Type Type {
            get { return type; }
        }

        /// <include file='doc\Provider.uex' path='docs/doc[@for="Provider.Properties"]/*'/>
        /// <devdoc>
        /// Gets the set of custom properties associated with the provider.
        /// </devdoc>        
        public XPathNavigator Properties {
            get { return properties; }
        }
        #endregion
    }
}
