using System;
using System.Collections;
using System.Collections.Specialized;
using System.Configuration;
using System.Xml;

namespace Microsoft.ApplicationBlocks.Common {
    /// <devdoc>
    /// The Configuration Section Handler for the provider sections of the configuration file.
    /// </devdoc>
    internal sealed class ProviderConfigHandler : IConfigurationSectionHandler {   
        #region Constructors
        /// <devdoc>
        /// Initialize a new instance of the <see cref="ProviderConfigHandler"/> class.
        /// </devdoc>
        public ProviderConfigHandler(){
        }
        #endregion

        #region IConfigurationSectionHandler Members
        /// <devdoc>
        /// Creates a Hashtable of providers and returns them indexed by system throws a ConfigurationException.
        /// </devdoc>        
        public Object Create(Object parent, Object input, XmlNode section) {			            
            Hashtable providers = new Hashtable(5);
            XmlNodeList nodeList = section.SelectNodes("provider");
            try {
                foreach(XmlNode node in nodeList) {
                    Provider provider = new Provider(node);
					if( providers.Contains(provider.Name.ToLower(System.Globalization.CultureInfo.CurrentCulture)) )
					{
						throw new ConfigurationException(String.Format(ResourceTable.GetString(ResourceTable.DuplicateProviderSpecified), provider.Name));
					}
                    providers.Add(provider.Name.ToLower(System.Globalization.CultureInfo.CurrentCulture), provider);
                }
            } catch (XmlException xmlEx) {
                throw new ConfigurationException(xmlEx.Message, xmlEx);
            } catch (System.Reflection.TargetInvocationException tEx) {
                throw new ConfigurationException(tEx.Message, tEx);
            } catch (System.TypeLoadException tlEx) {
                throw new ConfigurationException(tlEx.Message, tlEx);
            } catch (System.IO.FileNotFoundException fnfEx) {
                throw new ConfigurationException(fnfEx.Message, fnfEx);
            }     
            return providers;
        }
        #endregion
    }
}
