//=============================================================================
// Copyright  2003 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//=============================================================================

using System;
using System.Collections;
using System.Collections.Specialized;
using System.Configuration;
using System.Diagnostics;
using System.Reflection;
using System.Security.Permissions;

namespace Microsoft.ApplicationBlocks.Common {
    /// <include file='doc\ProviderManagerBase.uex' path='docs/doc[@for="ProviderManagerBase"]/*'/>        
    /// <devdoc>
    /// Base class for providers to read the configuration section of the applicaiton configuration file.
    /// </devdoc>    
    public abstract class ProviderManagerBase : MarshalByRefObject {
        /// <include file='doc\ProviderManagerBase.uex' path='docs/doc[@for="ProviderManagerBase.ProviderManagerBase"]/*'/>        
        protected ProviderManagerBase() {			
        }

        /// <include file='doc\ProviderManagerBase.uex' path='docs/doc[@for="ProviderManagerBase.GetProvider"]/*'/> 
        /// <devdoc>
        /// Get the provider from the <paramref name="configSection"/> for the <paramref name="providerName"/>.
        /// </devdoc>               
        [PermissionSet(SecurityAction.Demand, Name="FullTrust")]
        protected IProvider GetProvider(String configSection, String providerName) { 
            String nameOfProviderToFetch = providerName.ToLower(System.Globalization.CultureInfo.CurrentCulture);
			IDictionary providers = ConfigurationSettings.GetConfig(configSection) as IDictionary;
            Debug.Assert(providers != null);
            if (providers == null) {
                throw new ConfigurationException(String.Format(ResourceTable.GetString(ResourceTable.ConfigSectionError), configSection));
            }
            Provider provider = providers[nameOfProviderToFetch] as Provider;
            if (provider == null) {
                throw new ConfigurationException(String.Format(ResourceTable.GetString(ResourceTable.NoProvider), providerName));
            }
            Debug.Assert(provider != null);
            try {
                IProvider providerType = Activator.CreateInstance(provider.Type) as IProvider;
                Debug.Assert(providerType != null);
                if (providerType == null) {
                    throw new ConfigurationException(String.Format(ResourceTable.GetString(ResourceTable.NoProvider), providerName));
                }
                providerType.Initialize(provider.Properties);
                return providerType;
            } catch (TargetInvocationException tiEx) {
                throw new ConfigurationException(tiEx.Message, tiEx);
            }
        }       
    }
}
