//=============================================================================
// Copyright  2003 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//=============================================================================

using System;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Resources;

namespace Microsoft.ApplicationBlocks.Common {
    /// <devdoc>
    /// A configuration manager for reading a resource table.
    /// </devdoc>
    sealed class ResourceTable : System.MarshalByRefObject {	
        #region Constructors       
        private ResourceTable() {
        }
        #endregion

        #region Static Methods
        /// <devdoc>
        /// Looks up a localizable String in the resources.
        /// </devdoc>        
        public static String GetString(String res) {   
            Debug.Assert(res != null && res.Length > 0);
            ResourceManager rm = new ResourceManager(String.Concat(typeof(ResourceTable).Namespace, ".ResourceTable"), Assembly.GetExecutingAssembly());
            return rm.GetString(res, System.Globalization.CultureInfo.CurrentCulture);
        }       
        #endregion		

        #region Lookup Strings              
        public const String NoProvider              = "1"; 
        public const String Name                    = "2";
        public const String Type                    = "3";
        public const String InvalidConifgInterface  = "4";
        public const String ValidationKey           = "5";
        public const String FormatFailed            = "6";
        public const String Validation              = "7";
        public const String InvalidBaseString       = "8";
        public const String InvalidAlg              = "9";
        public const String PlainTextBuffErr        = "10";
        public const String MarshalDataErr          = "11";
        public const String EntropyBuffErr          = "12";
        public const String MarshalEntropyErr       = "13";
        public const String EncryptionFailed        = "14";
        public const String CipherTextErr           = "15";
        public const String DecryptionFailed        = "16";
        public const String ConfigSectionError      = "17";
		public const String DuplicateProviderSpecified	= "18";
		public const String DuplicateCryptographicInfoSpecified	= "18";
		#endregion
    }
}
