using System;
using System.Reflection;

using Microsoft.ApplicationBlocks.Cache;

namespace Microsoft.ApplicationBlocks.SmartClient.Offline
{
	/// <summary>
	/// A data structure used to store the metadata for isolated storage.
	/// </summary>
	[Serializable]
    class MetadataItem
	{
        private ICacheItemExpiration[] expirations;
        private CacheItemPriority priority;		
		private Type type;
		private string methodName;		

		public MetadataItem(ICacheItemExpiration[] expirations, CacheItemPriority priority, CacheItemRemovedCallback callback) {
            this.expirations = expirations;
            this.priority = priority;
			if (callback != null) {
				if (callback.Method.IsStatic) {
					this.methodName = callback.Method.Name;
					this.type = callback.Method.DeclaringType;
				}				
			}			
		}

        public ICacheItemExpiration[] Expirations {
            get { return expirations; }
        }

        public CacheItemPriority Priority {
            get { return priority; }
        }

		public CacheItemRemovedCallback Callback {
			get { 
				if ((type != null) && (methodName.Length > 0)) {
					Delegate d = Delegate.CreateDelegate(typeof(CacheItemRemovedCallback), type, methodName);
					CacheItemRemovedCallback callback = new CacheItemRemovedCallback((CacheItemRemovedCallback)d);
					return callback;
				}
				return null;	
			}
		}
	}
}
