//=============================================================================
// Copyright  2004 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//=============================================================================
using System;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Resources;

namespace Microsoft.ApplicationBlocks.SmartClient.Offline
{
	/// <summary>
	/// This class implements the globalization strategy for these providers by performing lookups for all strings
	/// </summary>
	public sealed class ProvidersResourceTable : System.MarshalByRefObject 
	{	
		private ProvidersResourceTable() 
		{
		}

		/// <summary>
		/// Does the lookup from an identifier to the actual string
		/// </summary>
		/// <param name="res">Key to use for lookup</param>
		/// <returns>String corresponding to the lookup key</returns>
		public static String GetString(String res) 
		{   
			Debug.Assert(res != null && res.Length > 0);
			ResourceManager rm = new ResourceManager(String.Concat(typeof(ProvidersResourceTable).Namespace, ".ProvidersResourceTable"), Assembly.GetExecutingAssembly());
			return rm.GetString(res, System.Globalization.CultureInfo.CurrentCulture);
		}       

		#region Lookup Strings      
		public const string ConnectionUnavailableMessage	=	"1";
		public const string PollingIntervalShouldBeNumeric	=	"2";
		public const string ConnectionManagerPollingIntervalMessage = "3";
		public const string QueueMessageFailedVerification = "4";
		public const string InvalidOrMissingAttributeInConfigFile = "5";
		public const string InvalidOrMissingElementInConfigFile = "5";
		public const string InvalidFieldFormatInConfigFile = "6";
		#endregion
	}
}
