//=============================================================================
// Copyright  2004 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//=============================================================================
using System;
using System.Collections;
using System.Xml.XPath;

using Microsoft.ApplicationBlocks.Common;

namespace Microsoft.ApplicationBlocks.SmartClient.Offline
{
	/// <summary>
	/// Provides the ability for the QueueManager to store its pending messages
	/// on an in-memory System.Collections.Queue. This class is provided as a starter only and should
	/// never be used in a production environment. It is not persistent and all messages will be lost
	/// if applicaition is shut down.
	/// </summary>
	public class InMemoryQueueStorageProvider : QueueStorageProvider
	{
		private Queue queuedMessages = new Queue();
			
		/// <summary>
		/// Getter property to retrieve the number of messages stored in the queue
		/// </summary>
		public override int Size { get { return queuedMessages.Count; }}

		/// <summary>
		/// Empty constructor for instance
		/// </summary>
		public InMemoryQueueStorageProvider()
		{
		}

		/// <summary>
		/// Constructor used for testing. Applications should never directly instantiate this class
		/// </summary>
		/// <param name="isEncrypted">Should queue messages be encrypted</param>
		/// <param name="isSigned">Should queue messages be signed</param>
		public InMemoryQueueStorageProvider(bool isEncrypted, bool isSigned)
		{
		}

		/// <summary>
		/// Flushes all queue messages off the queue
		/// </summary>
		public override void Flush()
		{
			queuedMessages.Clear();
		}

		/// <summary>
		/// Responsible for returning the next object off the in-memory queue as an array of bytes
		/// </summary>
		/// <returns></returns>
		override protected byte[] DoDequeue()
		{
			return queuedMessages.Dequeue() as byte[];
		}

		/// <summary>
		/// Responsible for enqueuing the specified array of bytes onto the in-memory queue
		/// </summary>
		/// <param name="objectStream"></param>
		override protected void DoEnqueue(byte[] objectStream)
		{
			queuedMessages.Enqueue(objectStream);
		}

		/// <summary>
		/// No implementation in this class
		/// </summary>
		/// <param name="parameters"></param>
		protected override void DoLocalSetup(XPathNavigator parameters)
		{
			;
		}

		/// <summary>
		/// No implementation in this class
		/// </summary>
		protected override void Initialize()
		{
			;
		}
	}
}
