//=============================================================================
// Copyright  2004 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//=============================================================================
using System;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;

namespace Microsoft.ApplicationBlocks.SmartClient.Offline
{
	/// <summary>
	/// Utility methods used is streaming objects to and from byte arrays
	/// </summary>
	/// 
	internal class StreamingUtils
	{
		/// <summary>
		/// Streams an object to a byte array
		/// </summary>
		/// <param name="objectToStream">Object to convert to byte array</param>
		/// <returns>Byte array representing the object</returns>
		public static byte [] ToBytes(object objectToStream)
		{
			using(MemoryStream itemStream = new MemoryStream())
			{
				new BinaryFormatter().Serialize(itemStream, objectToStream);
				return itemStream.ToArray();
			}
		}

		/// <summary>
		/// Converts an array of bytes back into an object
		/// </summary>
		/// <param name="msgBytes">byte array representing the object</param>
		/// <returns>Reconstituted object</returns>
		public static object ToObject(byte[] msgBytes)
		{
			using(MemoryStream msgStream = new MemoryStream(msgBytes))
			{
				return new BinaryFormatter().Deserialize(msgStream);
			}
		}
	}
}
