//=============================================================================
// Copyright  2004 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//=============================================================================
using System;
using Microsoft.ApplicationBlocks.SmartClient.Offline;

namespace Microsoft.ApplicationBlocks.SmartClient.Offline
{
	/// <summary>
	/// Responsible for construction of ConnectionManager and exposing the instantiated object as
	/// a property
	/// </summary>
	public class ConnectionManagerBuilder
	{
		private ConnectionManager connectionManager;

		/// <summary>
		/// Returns the ConnectionManager instance
		/// </summary>
		public ConnectionManager ConnectionManager { get { return connectionManager; }}

		/// <summary>
		/// Creates the ConnectionManager component. Creates the internal pieces and wires them together
		/// </summary>
		public ConnectionManagerBuilder()
		{
			ConnectionManagerConfigurator configurator = new ConnectionManagerConfigurator();
			connectionManager = new ConnectionManager(configurator.DetectionStrategy, configurator.PollInterval);
		}

		/// <summary>
		/// Starts the ConnectionManager. This method will begin the polling cycles for any ConnectionDetectionStrategies.
		/// </summary>
		public void Start()
		{
			connectionManager.Start();
		}

		/// <summary>
		/// Stops the ConnectionManager. This method may potentially block for several seconds as the
		/// polling threads die and are joined.
		/// </summary>
		public void Stop()
		{
			connectionManager.Stop();
		}
	}
}
