//=============================================================================
// Copyright  2004 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//=============================================================================
using System;

namespace Microsoft.ApplicationBlocks.SmartClient.Offline
{
	/// <summary>
	/// Creates the Executur and exposes the instance as a property
	/// </summary>
	public class ExecutorBuilder
	{
		private ICommandProcessor commandProcessor;
		private Executor executor;

		/// <summary>
		/// Exposes the Executor instance as a property
		/// </summary>
		public Executor Executor { get { return executor; }}

		/// <summary>
		/// Constructor for the Executor. It creates the embedded instances and wires them together.
		/// </summary>
		/// <param name="resultConsumer">The object responsible for processing results after a Payload has been executed.
		/// This is usually the ServiceAgentManager.</param>
		/// <param name="queueMessageProducer">Object responsible for providing QueueMessages to be processed</param>
		public ExecutorBuilder(IResultConsumer resultConsumer, IQueueMessageProducer queueMessageProducer)
		{
			commandProcessor = new SimpleCommandProcessor(resultConsumer);
			executor = new Executor(queueMessageProducer, commandProcessor);
		}
	}
}
