//=============================================================================
// Copyright  2004 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//=============================================================================
using System;

namespace Microsoft.ApplicationBlocks.SmartClient.Offline
{
	/// <summary>
	/// Responsible for construction of QueueManager and exposing the instantiated object as
	/// a property
	/// </summary>
	public class QueueManagerBuilder
	{
		private QueueManager queueManager;
		private IQueueStorageProvider queueStorageProvider;

		/// <summary>
		/// Getter property to return the QueueManager object
		/// </summary>
		public QueueManager QueueManager { get { return queueManager; }}

		/// <summary>
		/// Getter property to return the QueueManager object as an IQueueMessageProducer. This property should be used
		/// by clients that want to retrieve QueueMessages from the QueueManager.
		/// </summary>
		public IQueueMessageProducer QueueMessageProducer { get { return queueManager; }}

		/// <summary>
		/// Getter property to return the QueueManager object as an IPayloadConsumer. This property should be used
		/// by clients that want to add Payloads to the QueueManager.
		/// </summary>
		public IPayloadConsumer PayloadConsumer { get { return queueManager; }}

		/// <summary>
		/// Constructor responsible for initializing instances of QueueManager and wiring together all its parts.
		/// </summary>
		public QueueManagerBuilder()
		{
			QueueManagerConfigurator configurator = new QueueManagerConfigurator();
			queueManager = new QueueManager(configurator.QueueStorageProvider);
			queueStorageProvider = configurator.QueueStorageProvider;
		}
	}
}
