//=============================================================================
// Copyright  2004 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//=============================================================================
using System;
using System.Configuration;
using System.Collections;
using System.Xml.XPath;
using Microsoft.ApplicationBlocks.Common;

namespace Microsoft.ApplicationBlocks.SmartClient.Offline
{
	/// <summary>
	/// This class is responsible for parsing the contents of the configuration file
	/// and making the appropriate pieces available through properties
	/// </summary>
	public class ConnectionManagerConfigurator : ProviderManagerBase
	{
		private IConnectionDetectionStrategy detectionStrategy;
		private int							 pollInterval = -1;

		/// <summary>
		/// Returns an instance of the IConnectionDetectionStrategy as defined in the configuration file. This method uses
		/// Lazy Initialization to create the first time it is requested, and caches the instance to be returned the next time
		/// the method is called.
		/// </summary>
		public IConnectionDetectionStrategy	DetectionStrategy 
		{
			get
			{
				CreateDetectionStrategy();
				return detectionStrategy;
			}
		}

		/// <summary>
		/// Returns the polling interval as defined in the configuration file. This method uses Lazy Initialization to
		/// get this value the first time it is requested. It is cached and returned each subsequent time it is called.
		/// </summary>
		public int PollInterval 
		{
			get
			{
				CreateDetectionStrategy();				
				pollInterval = detectionStrategy.PollInterval;
	
				return pollInterval;
			}
		}

		private void CreateDetectionStrategy()
		{
			if(detectionStrategy == null)
			{
				IProvider detectionStrategyProvider = GetProvider("ConnectionManagerProviders", "connectionDetectionStrategy");
				detectionStrategy = detectionStrategyProvider as IConnectionDetectionStrategy;
			}
		}
	}
}
