//=============================================================================
// Copyright  2004 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//=============================================================================
using System;
using Microsoft.ApplicationBlocks.ExceptionManagement;

namespace Microsoft.ApplicationBlocks.SmartClient.Offline
{
	/// <summary>
	/// Common event handling and broadcasting code used throughout block
	/// </summary>
	public sealed class EventUtils
	{
		private EventUtils() {}

		/// <summary>
		/// Generic method used to broadcast events when needed. This method will attempt to send the event to each registered
		/// listener, even if any particular event throws an exception
		/// </summary>
		/// <param name="eventToBroadcast">Event to be broadcast</param>
		/// <param name="eventSender">Object that sent the event</param>
		/// <param name="eventData">EventArgs to be broadcast with the event</param>
		public static void BroadcastEvent(Delegate eventToBroadcast, object eventSender, EventArgs eventData)
		{
			if(eventToBroadcast != null) 
			{
				object [] args = new object[] { eventSender, eventData };
				foreach(Delegate callbackDelegate in eventToBroadcast.GetInvocationList())
				{
					try
					{
						callbackDelegate.DynamicInvoke(args);
					}
					catch(Exception ex)
					{
						ExceptionManager.Publish(ex);
					}
				}
			}
		}
	}
}
