//=============================================================================
// Copyright  2004 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//=============================================================================
using System;
using System.Reflection;

namespace Microsoft.ApplicationBlocks.SmartClient.Offline
{
	/// <summary>
	/// The SimpleCommandProcessor will execute the QueueMessage passed to it. It will dynamically invoke the online proxy method
	/// needed to carry out the message and will return the results to the result consumer.
	/// </summary>
	public class SimpleCommandProcessor : ICommandProcessor
	{
		private IResultConsumer resultConsumer;

		/// <summary>
		/// Property to get and set the ResultConsumer. This method is protected to allow for alterations during testing.
		/// Applications should not call the set property.
		/// </summary>
		protected IResultConsumer ResultConsumer
		{
			get { return resultConsumer; }
			set { resultConsumer = value; }
		}

		/// <summary>
		/// Constructor for instance.
		/// </summary>
		/// <param name="resultConsumer">Target for results of message execution</param>
		public SimpleCommandProcessor(IResultConsumer resultConsumer)
		{
			if(resultConsumer == null)
			{
				throw new ArgumentNullException("resultConduit", ResourceTable.GetString(ResourceTable.ExecutorArgumentNullMessage));
			}
			this.resultConsumer = resultConsumer;
		}

		/// <summary>
		/// Executes the given QueueMessage. It is expected that none of the methods called in this method throws an exception, as that
		/// would prevent the results or error from being returned correctly.
		/// </summary>
		/// <param name="message">Message to be executed</param>
		virtual public void ExecuteMessage(QueueMessage message)
		{
			Payload payload = message.MessagePayload;

			OnlineProxyContext methodToExecute = payload.MethodToExecute;
			methodToExecute.InvokeMethodThroughReflection(payload);
			
			ServiceAgentContext resultCallbackTarget = payload.ResultCallbackTarget;
			resultConsumer.ReturnDataToCaller(payload);
		}
	}
}
