//=============================================================================
// Copyright  2004 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//=============================================================================
using System;
using System.Xml.XPath;
using System.Xml;
using System.Collections;
using System.Configuration;

using Microsoft.ApplicationBlocks.Common;

namespace Microsoft.ApplicationBlocks.SmartClient.Offline
{
	/// <summary>
	/// This class is an implementation of IConfigurationSectionHandler that specifically handles provider
	/// sections that have several providers, only one of which is enabled at any time. It will perform much of
	/// the same functionality as the common block's ProviderConfigHandler.
	/// </summary>
	public class MultiProviderConfigHandler : IConfigurationSectionHandler
	{
		/// <summary>
		/// Parses the given XmlNode to find the one provider node that is enabled.
		/// </summary>
		/// <param name="parentSettings">Not used</param>
		/// <param name="webSettingsNotUsed">Not used</param>
		/// <param name="configNode">Providers node from application configuration file.</param>
		/// <returns></returns>
		public object Create(object parentSettings, object webSettingsNotUsed, XmlNode configNode)
		{
			Hashtable providerDictionary = new Hashtable(5);

			XmlNode enabledProviderNode = GetEnabledProviderNodeFromXml(configNode);
			Provider provider = CreateProvider(enabledProviderNode);

			providerDictionary.Add(provider.Name.ToLower(System.Globalization.CultureInfo.CurrentCulture), provider);

			return providerDictionary;
		}

		/// <summary>
		/// Creates new provider using the common block's Provider class
		/// </summary>
		/// <param name="enabledProviderNode"></param>
		/// <returns></returns>
		private Provider CreateProvider(XmlNode enabledProviderNode)
		{
			try
			{
				return new Provider(enabledProviderNode);
			}
			catch(Exception e)
			{
				throw new ConfigurationException(e.Message, e);
			}
			
		}

		/// <summary>
		/// Searches through the given Xml node using XPath to find the single enabled provider node
		/// </summary>
		/// <param name="configNode"></param>
		/// <returns></returns>
		private XmlNode GetEnabledProviderNodeFromXml(XmlNode configNode)
		{
			XPathNavigator configNavigator = configNode.CreateNavigator();
			XPathNodeIterator configIter = configNavigator.Select("//provider[@enabled='true']");
			if(configIter.Count != 1)
			{
				throw new ConfigurationException(ResourceTable.GetString(ResourceTable.MultiProviderConfigHandlerEnablingMessage));
			}
			configIter.MoveNext();

			return ((IHasXmlNode)configIter.Current).GetNode();
		}
	}
}
