//=============================================================================
// Copyright  2004 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//=============================================================================
using System;

namespace Microsoft.ApplicationBlocks.SmartClient.Offline
{
	/// <summary>
	/// This interface defines the provider that the QueueManager uses to
	/// store the QueueMessages onto the Queue. It is assumed that there will 
	/// be derived classes to use IsolatedStorage, MSMQ, and MSDE/SQLServer
	/// </summary>
	public interface IQueueStorageProvider
	{
		/// <summary>
		/// Getter property to return the number of messages currently stored
		/// </summary>
		/// <value>Number of messages stored</value>
		int Size { get; }

		/// <summary>
		/// Stores a QueueMessage into the eventual data sink
		/// </summary>
		/// <param name="queueMessage">Message to store</param>
		void Enqueue(QueueMessage queueMessage);

		/// <summary>
		/// Retrieves a QueueMessage from the data sink
		/// </summary>
		/// <returns>QueueMessage retrieved from data sink</returns>
		QueueMessage Dequeue();

		/// <summary>
		/// Clears all messages from data sink. Use with caution, as this will remove all messages.
		/// </summary>
		void Flush();
	}
}
