//=============================================================================
// Copyright  2004 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//=============================================================================
using System;

namespace Microsoft.ApplicationBlocks.SmartClient.Offline
{
	/// <summary>
	/// The FailsafeServiceAgent is the path of last resort to get messages back to the application in the event of some
	/// failure during message execution. It exposes an event that the application can register with to be called back when an error
	/// does occur. This service agent is intended to be used only when the original service agent is no longer available
	/// and there is no other way to return an error to the application. It is not meant to be used as the standard path back 
	/// for error results. It is to the designer's advantage to code and deploy their own service agent for handling the results
	/// from a successful call or the errors from an unsuccessful one, as that provides application-specific handling for the appropriate
	/// context.
	/// </summary>
	public class FailsafeServiceAgent : ServiceAgent
	{
		/// <summary>
		/// Event for the application to register with to be notified of errors that occured during message execution
		/// </summary>
		public event MethodExecutionFailureReportEventHandler ErrorEvent;

		/// <summary>
		/// This method broadcasts the event defined above to all registered listeners.
		/// </summary>
		/// <param name="payloadWithError">Payload containing the errors</param>
		override public void ReportError(Payload payloadWithError) 
		{
			MethodExecutionFailureReportEventArgs args = new MethodExecutionFailureReportEventArgs(payloadWithError);
			EventUtils.BroadcastEvent(ErrorEvent, this, args);
		}
	}
}
