//=============================================================================
// Copyright  2004 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//=============================================================================
using System;

namespace Microsoft.ApplicationBlocks.SmartClient.Offline
{
	/// <summary>
	/// Base class from which all ServiceAgents derive. Provides them with a unique Guid per instance and
	/// registers them with the ServiceAgentRegister.
	/// </summary>
	public abstract class ServiceAgent
	{
		private static ServiceAgentRegistry agentRegistry;

		private Guid guid = Guid.NewGuid();

		/// <summary>
		/// Protected constuctor that all derived ServiceAgents call. This constructor registers the instance with the registry.
		/// </summary>
		protected ServiceAgent()
		{
			if(agentRegistry != null)
				agentRegistry.Register(this);	
		}

		/// <summary>
		/// Called to set the ServiceAgentRegistry during construction. This is public to facilitate testing. Client code
		/// should not call this method
		/// </summary>
		/// <param name="theAgentRegistry">ServiceAgentRegistry instance</param>
		public static void SetServiceAgentRegistry(ServiceAgentRegistry theAgentRegistry)
		{
			agentRegistry = theAgentRegistry;	
		}

		/// <summary>
		/// Get the guid associated with this ServiceAgent instance
		/// </summary>
		/// <value>GUID of the Service Agent</value>
		virtual public Guid Guid { get { return guid; }}

		/// <summary>
		/// Generic method to report errors. Subclasses must implement this method to perform error
		/// handling and recovery from a failed message.
		/// </summary>
		/// <param name="payload">The payload that was being executed when the error occured. Error information is inside the payload</param>
		abstract public void ReportError(Payload payload);
	}
}
