//=============================================================================
// Copyright  2004 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//=============================================================================
using System;
using System.Reflection;

namespace Microsoft.ApplicationBlocks.SmartClient.Offline
{
	/// <summary>
	/// This class represents the information needed to identify which method to invoke in a particular
	/// Service Agent. Currently this class is trivial, holding only the method name. The design decision
	/// reached to continue using this class balanced the additional work of the programmer in having to create
	/// an instance of this class just to wrap a method name with the extra communication value added by having
	/// a class to wrap that name and expalain what it is. It is more work, but it communicates more clearly,
	/// and communication trumps fewest classes in the 4 Rules of Simplicity. 
	/// </summary>
	/// 
	[Serializable()]
	public class ServiceAgentContext
	{
		private string methodToInvoke;

		/// <summary>
		/// Returns the name of the method to invoke on the ServiceAgent-derived class
		/// </summary>
		/// <value>Method to be invoked</value>
		public string MethodToInvoke { get { return methodToInvoke; }}

		/// <summary>
		/// Constructor for instance
		/// </summary>
		/// <param name="methodToInvoke">Name of the method to invoke through reflection</param>
		public ServiceAgentContext(string methodToInvoke)
		{
			this.methodToInvoke = methodToInvoke;
		}
	}
}
