//=============================================================================
// Copyright  2004 Microsoft Corporation
// All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
// FITNESS FOR A PARTICULAR PURPOSE.
//=============================================================================
using System;
using System.Collections;
using System.Reflection;
using System.Threading;

using Microsoft.ApplicationBlocks.ExceptionManagement;

namespace Microsoft.ApplicationBlocks.SmartClient.Offline
{
	/// <summary>
	/// The ServiceAgentManager manages the registry of ServiceAgents and maintains the algorithm used to
	/// inform the ServiceAgent of the success or failure of a method execution
	/// </summary>
	public class ServiceAgentManager : IResultConsumer
	{
		private ServiceAgentRegistry agentRegistry;

		/// <summary>
		/// Constructor for instance
		/// </summary>
		/// <param name="agentRegistry">ServiceAgentRegistry instance</param>
		public ServiceAgentManager(ServiceAgentRegistry agentRegistry)
		{
			this.agentRegistry = agentRegistry;
		}

		/// <summary>
		/// Responsible for returning success or failure notification to the ServiceAgent. The ServiceAgent to which the
		/// results are returned is identified by the Payload.ServiceAgentGuid. If this specific ServiceAgent is not found, 
		/// then the FailsafeServiceAgent instance is returned from the ServiceAgentRegistry, and it is only used to return failures,
		/// should any exist.
		/// </summary>
		/// <param name="payload">Payload containing the ServiceAgentGuid</param>
		public void ReturnDataToCaller(Payload payload)
		{ 
			ServiceAgent serviceAgent = agentRegistry.Find(payload.ServiceAgentGuid);
			if(serviceAgent != null)
			{				
				ThreadPoolInvoker invoker = new ThreadPoolInvoker(serviceAgent, payload);
				if(payload.Success) 
				{
					invoker.ReturnResults();
				}
				else
				{
					invoker.ReturnFailure();
				}		
			}
		}
	}
}
