using System;
using System.Windows.Forms;
using System.Threading;
using Microsoft.Win32;
using Microsoft.ApplicationBlocks.ExceptionManagement;
using System.Reflection;
using System.Diagnostics;
using System.Text.RegularExpressions;
using System.Net;
using System.Security;
using System.IO;
using System.Configuration;

namespace Northwind
{
	/// <summary>
	/// Summary description for MainClass.
	/// </summary>
	public class MainClass
	{
		private static AutoUpdate _autoUpdate = null;

		public static void Main(string[] args)
		{
			try
			{
				if(Offline.IsOnline)
				{
					//DEMO 1: Try to login
					if(!Login())
					{
						Application_ApplicationExit(null, null);
						return;
					}
					string setting = ConfigurationSettings.AppSettings[
						"enableAppUpdater"];
					bool Switch = false;
					if(String.Empty != setting && null != setting)
						Switch = bool.Parse(setting);
	                
					if(Switch)
					{
						_autoUpdate = new AutoUpdate();
						_autoUpdate.InitializeAutoUpdate();
					}
				}
				Application.ApplicationExit += 
					new EventHandler(Application_ApplicationExit);

				Application.ThreadException += new ThreadExceptionEventHandler(
					MainUIThreadExceptionHandler);
				System.Windows.Forms.Application.Run(
					new NorthwindMDIMain());
			}
			catch(Exception e)
			{
				MessageBox.Show(e.ToString());
			}
		}
		public static void Application_ApplicationExit(object sender,
			EventArgs e)
		{
			Offline.Stop();
			if(null!=_autoUpdate)
				_autoUpdate.StopUpdater();
		}

		#region MainUIThreadExceptionHandler
		
		public static void MainUIThreadExceptionHandler(
			Exception e)
		{
			MainUIThreadExceptionHandler(null, new System.Threading.
				ThreadExceptionEventArgs(e));
		}

		public static void MainUIThreadExceptionHandler(object
			sender, ThreadExceptionEventArgs t)
		{

			ExceptionManager.Publish(t.Exception);

			//Tell the user a boo-boo occurred...
			MessageBox.Show(
				"Northwind Orders has encountered the following "
				+ " problem:" + (char)13
				+ (char)13 + t.Exception.Message
				+ (char)13 + (char)13 
				+ "Please contact your administrator or the "
				+ "Northwind Help desk at "
				+ (char)13 + (char)13 +"1-855-555-5555."
				+ (char)13 + (char)13 
				+ "We apologize for the inconvenience."
				, "Northwind Orders Encountered a Problem"
				, MessageBoxButtons.OK
				, MessageBoxIcon.Error
				, MessageBoxDefaultButton.Button1 
				, MessageBoxOptions.ServiceNotification);		
		}

		#endregion

		public static void WriteInstalledVersion()
		{
			RegistryAccess ra = new RegistryAccess();

			ra.WriteRegistryValue("Installed Version"
				, Assembly.GetExecutingAssembly().GetName().
				Version.ToString());
		}

		//TODO:  Add authentication logic...
		private static bool Login()
		{
			bool ret=false;
			frmLogin loginForm = new frmLogin();
			DialogResult res = DialogResult.OK;
			while(DialogResult.Cancel != res)
			{
				loginForm.TopMost=true;
				res = loginForm.ShowDialog();
				if(DialogResult.OK==res)
				{
					string hashedPassword = Northwind.
						Utility.Cryptography.ComputeHash(
						loginForm.Password);
					NorthwindCredentials creds =  new NorthwindBusinessLogicFacade().
						Authenticate(
						loginForm.UserName
						, hashedPassword);
					if(null==creds)
					{
						MessageBox.Show("Authentication denied."
							, "Invalid Credentials"
							, MessageBoxButtons.OK
							, MessageBoxIcon.Warning);
					}
					else
					{
						//if we logged in, assign the principal to the
						//threads CurrentPrincipal property
						System.Threading.Thread.CurrentPrincipal = creds;
						ret = true;
						break;
					}
				}
			}
			return ret;
		}
	}
}

