using System;
using System.Data;
using System.Collections;
using System.Collections.Specialized;
using System.IO;
using System.Diagnostics;
using System.Reflection;
using System.Runtime.Serialization.Formatters.Binary;
using System.Threading;

namespace Northwind
{
	/// <summary>
	/// Summary description for NorthwindBusinessLogicFacade.
	/// </summary>
	public class NorthwindBusinessLogicFacade : IDisposable
	{
		private Cache _cache = new Cache();

		public NorthwindBusinessLogicFacade()
		{
		} 

		~NorthwindBusinessLogicFacade()
		{
			Dispose();
		}

		public NorthwindCredentials Authenticate(string UserName
			,string Password)
		{
			if(Offline.IsOnline)
			{
				NorthwindCredentials creds =
					new NorthwindBusinessLogicLayer.NorthwindBusinessLogicLayer()
					.Authenticate(UserName, Password);
				//p-lunk it into cache...
				_cache.Add("credentials", creds);
				return creds;
			}
			else
			{
				//get it from cache...
				return (NorthwindCredentials)
					_cache["credentials"];
			}
		}

		public bool IsCacheAvailable()
		{
			return _cache.IsCacheAvailable;	
		}
		
		public dsCustomers RetrieveCustomer(string ID)
		{
			dsCustomers ret=null;
			if(Offline.IsOnline)
			{
				//get it from the web service
				ret=new NorthwindBusinessLogicLayer.NorthwindBusinessLogicLayer()
					.RetrieveCustomer(ID);
			}
			else
			{
				ret = new dsCustomers();
				dsCustomers.CustomersRow[] rows = (dsCustomers.CustomersRow[])((dsCustomers)_cache[
					"dsCustomers"]).Tables[0].Select("CustomerID='" + ID + "'");
				ret.Tables[0].ImportRow((DataRow)rows[0]);
			}
			return ret;
		}

		public int RetrieveCustomerCount()
		{
			if(Offline.IsOnline)
			{
				//get it from the web service
				return new NorthwindBusinessLogicLayer.NorthwindBusinessLogicLayer()
					.RetrieveCustomerCount();
			}
			else
			{
				return (int)((DataSet)_cache
					["CustomerList"]).Tables[0].Rows.Count;				
			}
		}

		public StringCollection RetrieveCustomerCountryList()
		{
			StringCollection ret = null;
			if(Offline.IsOnline)
			{
				//Get it from the web service...
				ret = new StringCollection();
				ret.AddRange(new NorthwindBusinessLogicLayer.NorthwindBusinessLogicLayer()
					.RetrieveCustomerCountryList());
				//plunk it into cache...
				_cache.Add("CustomerCountryList",ret);
			}
			else
			{
				ret = (StringCollection)_cache["CustomerCountryList"];
			}
			return ret;
		}
		
		public DataSet RetrieveCustomerList()
		{
			DataSet ret = null;
			if(Offline.IsOnline)
			{
				//get it from the web service
				ret = new NorthwindBusinessLogicLayer.NorthwindBusinessLogicLayer()
					.RetrieveCustomerList();
				//put it in the local cache...
				_cache.Add("CustomerList",ret);
			}
			else
			{
				ret = (DataSet)_cache["CustomerList"];
			}
			return ret;
		}
		
		public StringCollection RetrieveCustomerRegionList()
		{
			StringCollection ret = new StringCollection();
			if(Offline.IsOnline)
			{
				//Get it from the webservice
				ret.AddRange(
					new NorthwindBusinessLogicLayer.NorthwindBusinessLogicLayer()
					.RetrieveCustomerRegionList());
				//add it to the cache...
				_cache.Add("CustomerRegionList",ret);
			}
			else
			{
				ret = (StringCollection)_cache["CustomerRegionList"];
			}
			return ret;
		}

		public dsCustomers RetrieveCustomers()
		{
			dsCustomers ret = null;
			if(Offline.IsOnline)
			{
				//Get it from the webservice
				ret = new NorthwindBusinessLogicLayer.
					NorthwindBusinessLogicLayer()
					.RetrieveCustomers();
				//add it to the cache...
				_cache.Add("dsCustomers",ret);
			}
			else
			{
				ret = (dsCustomers)_cache["dsCustomers"];
			}
			return ret;
		}
		
		public dsEmployees RetrieveEmployees()
		{
			dsEmployees ret = null;
			if(Offline.IsOnline)
			{
				//get it from the web service
				ret = (dsEmployees)new NorthwindBusinessLogicLayer.
					NorthwindBusinessLogicLayer().RetrieveEmployees();
				//plunk it into the local cache...
				_cache.Add("dsEmployees",ret);
			}
			else
			{
				ret = (dsEmployees)_cache["dsEmployees"];
			}
			return ret;
		}

		public int RetrieveOrderCount()
		{
			if(Offline.IsOnline)
			{
				//get it from the web service...
				return new NorthwindBusinessLogicLayer.NorthwindBusinessLogicLayer()
					.RetrieveOrderCount();
			}
			else
			{
				//get it from cache
				return (int)((dsOrdersOrderDetails)_cache[
					"dsOrdersOrderDetails"]).Tables[1].Rows.Count;
			}
		}
	
		public dsOrders RetrieveOrders(int EmployeeID
			, string CustomerID)
		{
			dsOrders ret = null;
			if(Offline.IsOnline)
			{
				//get it directly from the service...
				ret = new NorthwindBusinessLogicLayer.
					NorthwindBusinessLogicLayer().RetrieveOrders(EmployeeID, CustomerID);
				//put it in the cache...
				_cache.Add("dsOrders",ret);
			}
			else
			{
				//Get it from the local cache
				ret = (dsOrders)_cache["dsOrders"];
			}
			return ret;
		}

		public dsOrdersOrderDetails RetrieveOrdersOrderDetails()
		{
			dsOrdersOrderDetails ret = null;

			if(Offline.IsOnline)
			{
				//Get it from the web service
				ret = (dsOrdersOrderDetails)new NorthwindBusinessLogicLayer.
					NorthwindBusinessLogicLayer().RetrieveOrdersOrderDetails();
				//plunk it into cache...
				_cache.Add("dsOrdersOrderDetails",ret);
			}
			else
			{
				ret = (dsOrdersOrderDetails)_cache["dsOrdersOrderDetails"];
			}
			return ret;
		}

		public dsProducts RetrieveProducts()
		{
			if(Offline.IsOnline)
			{
				//get it directly from the service...
				dsProducts ret = new NorthwindBusinessLogicLayer.
					NorthwindBusinessLogicLayer().RetrieveProducts();
				//put it in the cache...
				_cache.Add("dsProducts",ret);
				return ret;
				//Update
			}
			else
			{
				//Get it from the local cache
				return (dsProducts)_cache["dsProducts"];
			}
		}

		public dsRegionTerritoryEmployeeCustomer RetrieveRegionTerritoryEmployeeCustomer()
		{
			dsRegionTerritoryEmployeeCustomer ret = null;
			if(Offline.IsOnline)
			{
				//get it from the webservice
				ret = new NorthwindBusinessLogicLayer.
					NorthwindBusinessLogicLayer().RetrieveRegionTerritoryEmployeeCustomer();
				//plunk it into the cache...
				_cache.Add("dsRegionTerritoryEmployeeCustomer",ret);
			}
			else
			{
				ret = (dsRegionTerritoryEmployeeCustomer)_cache[
					"dsRegionTerritoryEmployeeCustomer"];
			}
			return ret;
		}

		public void UpdateCustomers(dsCustomers custs)
		{
			new NorthwindBusinessLogicLayer.NorthwindBusinessLogicLayer()
				.UpdateCustomers(custs);
		}

		public void UpdateEmployees(dsEmployees emps)
		{
			new NorthwindBusinessLogicLayer.NorthwindBusinessLogicLayer()
				.UpdateEmployees(emps);
		}
		
		public void UpdateOrdersOrderDetails(
			dsOrdersOrderDetails orders)
		{
			new NorthwindBusinessLogicLayer.NorthwindBusinessLogicLayer()
				.UpdateOrdersOrderDetails(
				orders);
		}
	
		public void UpdateProducts(dsProducts
			Products)
		{
			new NorthwindBusinessLogicLayer.NorthwindBusinessLogicLayer()
				.UpdateProducts(Products);
		}

		public void RefreshCache()
		{
			if(Offline.IsOnline)
			{
				Thread t = new Thread(new ThreadStart(AsynchRefreshCache));
				t.IsBackground = true;
				t.Priority = ThreadPriority.BelowNormal;
				t.Start();
			}
		}

		private void AsynchRefreshCache()
		{
			try
			{
				this.RetrieveCustomers();
				this.RetrieveCustomerCountryList();
				this.RetrieveCustomerList();
				this.RetrieveCustomerRegionList();
				this.RetrieveEmployees();
				this.RetrieveOrdersOrderDetails();
				this.RetrieveProducts();
				this.RetrieveRegionTerritoryEmployeeCustomer();
				_cache.CommitCache();
			}
			catch(Exception e)
			{
				//Marshal the exception back into the main UI thread
				System.Threading.ThreadExceptionEventHandler teeh =
					new System.Threading.ThreadExceptionEventHandler(
					MainClass.MainUIThreadExceptionHandler);
				teeh.BeginInvoke(this,
					new System.Threading.
					ThreadExceptionEventArgs(e)
					,null
					,null);
			}
		}
		#region IDisposable Members

		bool isDisposed = false;

		public void Dispose() 
		{
			Dispose(false);
			// This object will be cleaned up by the Dispose method.
			// Therefore, you should call GC.SupressFinalize to
			// take this object off the finalization queue 
			// and prevent finalization code for this object
			// from executing a second time.
			GC.SuppressFinalize(this);
		}

		public void Dispose(bool Disposing)
		{
			if(!isDisposed)
			{
				_cache.Dispose();
			}
			isDisposed=true;
		}

		#endregion

		private class Cache:IDisposable 
		{
			private Hashtable _Cache = new Hashtable();
			private bool _isDirty = false;
			private static object _locker = new object();

			public bool IsDirty
			{
				get
				{
					return _isDirty;
				}
			}

			public bool IsCacheAvailable
			{
				get
				{
					return File.Exists(GetFileName());
				}
			}

			public Cache()
			{
				string fileName = GetFileName();
				if(File.Exists(fileName))
				{
					lock(_locker)
					{
						FileStream fs = new FileStream(
							fileName
							, FileMode.Open 
							, FileAccess.Read);
						BinaryFormatter bf = new BinaryFormatter();
						_Cache = (Hashtable)bf.Deserialize(fs);
						fs.Close();
					}
				}
			}

			~Cache()
			{
				Dispose(false);
			}

			private string GetFileName()
			{
				return Path.Combine(
					Path.GetDirectoryName(Assembly.GetEntryAssembly().Location)
					, "OfflineCache.bin");
			}

			public bool Contains(string Key)
			{
				return _Cache.Contains(Key);
			}

			public void Add(string Key, object Item)
			{
				if(Contains(Key))
					UpdateItem(Key, Item);
				else
					_Cache.Add(Key, Item);
				_isDirty = true;
			} 

			public object this[string Key]
			{
				get
				{
					return _Cache[Key];
				}
			}

			public void Remove(string Key)
			{
				_Cache.Remove(Key);
				_isDirty = true;
			}

			public void UpdateItem(string Key, object Item)
			{
				_Cache[Key] = Item;
				_isDirty = true;
			}

			public void PurgeCache()
			{
				_Cache.Clear(); 
				DeleteCache();
				_isDirty = true;
			}

			private void DeleteCache()
			{
				string fileName = GetFileName();
				if(File.Exists(fileName))
				{
					lock(_locker)
					{
						File.Delete(fileName);
					}
				}
			}

			public void CommitCache()
			{
				if(IsDirty)
				{
					string fileName = GetFileName();
					if(_Cache.Count > 0)
					{
						lock(_locker)
						{
							FileStream fs = new FileStream(
								fileName
								, FileMode.Create 
								, FileAccess.ReadWrite);
							BinaryFormatter bf = new BinaryFormatter();
							bf.Serialize(fs,_Cache);
							fs.Close();
						}
					}
					else
					{
						DeleteCache();
					}
					_isDirty = false;
				}
			}

			#region IDisposable Members

			bool isDisposed = false;

			public void Dispose() 
			{
				Dispose(false);
				// This object will be cleaned up by the Dispose method.
				// Therefore, you should call GC.SupressFinalize to
				// take this object off the finalization queue 
				// and prevent finalization code for this object
				// from executing a second time.
				GC.SuppressFinalize(this);
			}

			private void Dispose(bool Disposing)
			{
				if(!isDisposed)
				{
					if(Disposing)
					{
						if(IsDirty)
						{
							if( _Cache.Count > 0)
							{
								CommitCache();
								_Cache.Clear();
							}
							else
								DeleteCache();
						}
					}
				}
				isDisposed=true;				
			}
			#endregion

		}
	}
}