using System;
using System.Diagnostics;
using System.Threading;
using System.Windows.Forms;
using Microsoft.ApplicationBlocks.SmartClient.Offline;

namespace Northwind
{
	/// <summary>
	/// Summary description for Offline.
	/// </summary>
	internal class Offline : IDisposable
	{
		private static ConnectionState _currentState = ConnectionState.Unknown;
		private static ManualResetEvent _notifyMe = new ManualResetEvent(false);
		private static bool _isFirstLogin = true;

		public static ConnectionState CurrentState
		{
			get
			{
				if(_isFirstLogin)
				{
					//not setup yet.  Wait for it...
					_notifyMe.WaitOne();
				}
				return _currentState;
			}
		}
		
		public static bool IsOnline
		{
			get
			{
				return CurrentState == ConnectionState.OnLine;
			}
		}

		static Offline()
		{
			// Register with Connection Manager for online and offline 
			// state changed events.
			AddOfflineStateChangeNotify(
				new ConnectionStateChangedEventHandler(
				ConnectionManagerConnectionStateChangedEvent));
			Start();
		}

		public static void Start()
		{
			OfflineBlockBuilder.Instance.Start();
		}

		public static void Stop()
		{
			OfflineBlockBuilder.Instance.Stop();
		}

		public static void GoOffline()
		{
			OfflineBlockBuilder.Instance.ConnectionManager.GoOffline();
		}

		public static void GoOnline()
		{
			OfflineBlockBuilder.Instance.ConnectionManager.GoOnline();
		}

		private static void ConnectionManagerConnectionStateChangedEvent(
			object sender, ConnectionStateChangedEventArgs args)
		{
			if(ConnectionState.OnLine == args.CurrentState)
			{
				if(_isFirstLogin)
				{
					_isFirstLogin = false;
					DialogResult dlgResult = MessageBox.Show(
						"The system has detected Online state, Do you want to work Offline?",
						"Connection Management Quick Start"
						,MessageBoxButtons.YesNo
						,MessageBoxIcon.Question); 
					if(DialogResult.Yes == dlgResult)
					{
						OfflineBlockBuilder.Instance.ConnectionManager.GoOffline();
						return;
					}					
				}
			}
			else
			{
				if(ConnectionState.OffLine == args.CurrentState)
				{
					//notify the outside world that we're now offline...
					
				}
			}

			_currentState = args.CurrentState;

			if(null!=_notifyMe)
				_notifyMe.Set();
		}

		public static void AddOfflineStateChangeNotify(
			ConnectionStateChangedEventHandler csceh)
		{
			if(null!=csceh)
				OfflineBlockBuilder.Instance.ConnectionManager.
					ConnectionStateChangedEvent += csceh;
		}

		#region IDisposable Members
		private bool _isDisposed=false;
		public void Dispose() 
		{
			if(_isDisposed) return;
			try
			{
				OfflineBlockBuilder.Instance.Stop();
				OfflineBlockBuilder.Instance.Dispose();
			}
			catch{}
			finally{_isDisposed=true;}
		}

		#endregion
	}
}
