using System;
using System.Security;
using System.Security.Principal;
using System.Collections.Specialized;
using System.Xml.Serialization;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;

namespace Northwind
{
	/// <summary>
	/// Summary description for NorthwindCredentials.
	/// </summary>
	/// 
	[Serializable]
	public sealed class NorthwindCredentials : IPrincipal
	{
		private System.Security.Principal.GenericIdentity _Identity;
		private StringCollection _roles = new StringCollection();
		private DateTime _Created;
		private bool _properlyInitted = false;

		//will be called by XML Serialization...
		public string Payload
		{
			get
			{
				CheckProperlyInitted();
				CheckExpired();
				MemoryStream ms = new MemoryStream();
				Stream s = new Utility.Cryptography("NorthwindKey"
					).WrapStreamForEncryption(ms);
				BinaryFormatter bf = new BinaryFormatter();
				bf.Serialize(s, _Identity);
				bf.Serialize(s, _roles);
				bf.Serialize(s, _Created);
				s.Close();
				return Convert.ToBase64String(ms.ToArray());
			}	
			set
			{
				MemoryStream ms = new MemoryStream(
					Convert.FromBase64String(value));
				Stream s = new Utility.Cryptography("NorthwindKey"
					).WrapStreamForDecryption(ms);
				BinaryFormatter bf = new BinaryFormatter();
				_Identity = (GenericIdentity)bf.Deserialize(s);
				_roles = (StringCollection)bf.Deserialize(s);
				_Created = (DateTime)bf.Deserialize(s);
				_properlyInitted = true;
				ms.Close();
			}
		}

		public NorthwindCredentials()
		{
			//Required by XML Serialization
		}

		public NorthwindCredentials(string UserName, string[] roles)
		{
			_Identity = new GenericIdentity(UserName
				, "NorthwindCredentials");
			if(null!=roles)
				_roles.AddRange(roles);
			_Created = DateTime.Now;
			_properlyInitted = true;
		}

		#region IPrincipal Members

		//XML Serialization will barf on this if we don't:
		[XmlIgnoreAttribute]
		public IIdentity Identity
		{
			get
			{
				CheckProperlyInitted();
				CheckExpired();
				return _Identity;
			}
		}

		public bool IsInRole(string role)
		{
			CheckProperlyInitted();
			CheckExpired();
			return _roles.Contains(role);
		}

		private void CheckProperlyInitted()
		{
			if(!_properlyInitted)
				throw new SecurityException(
					"Principal not properly initialized.  Might be spoofed.");
		}

		private void CheckExpired()
		{
			if(_Created.AddHours(48) <= DateTime.Now)
				throw new SecurityException("Credentials have expired.");
		}
		#endregion
	}
}
