using System;
using System.IO;
using System.Security.Cryptography;
using System.Text;

namespace Utility
{
	/// <summary>
	/// Summary description for Encryption.
	/// </summary>
	//Encapsulates the functionality of encrypting with
	//the DES encryption algorithm and hashing with MD5
	public class Cryptography
	{
		//Required for DES stream-based Encryption
		private byte[] _IV;
		private byte[] _key;

		public Cryptography(string Key)
		{
			CreateDESKeyAndVector(Key
				,ref _IV 
				,ref _key);
		}

		private void CreateDESKeyAndVector(string Key
			, ref byte[] KeyOut
			, ref byte[] InitializationVectorOut)
		{

			const int KEY_SIZE_IN_BYTES = 8; //64 bits

			// Byte array to hold key        
			byte[] arrByte = Encoding.Unicode.GetBytes(Key);

			//Get the 16-byte hash value of the password
			SHA1CryptoServiceProvider objSHACryptoServiceProv =
				new SHA1CryptoServiceProvider();

			byte[] arrHash = objSHACryptoServiceProv.ComputeHash( 
				arrByte);

			KeyOut = (byte[])Array.CreateInstance(typeof(byte),(KEY_SIZE_IN_BYTES));
			InitializationVectorOut = (byte[])Array.CreateInstance(typeof(byte),
				(KEY_SIZE_IN_BYTES));

       
			for(int intCtr = 0;intCtr < KEY_SIZE_IN_BYTES; intCtr++)
			{
				//stick the low-order 8 bytes of the hash into the key
				KeyOut[intCtr] = arrHash[intCtr];
				//stick the high-order 8 bytes of the hash into the initialization vector
				InitializationVectorOut[intCtr] = arrHash[intCtr 
					+ KEY_SIZE_IN_BYTES];
			}
		}

		public string EncryptEncodeString(string EncryptAndEncodeThis)
		{

			//convert our input string to a byte array
			byte[] inputByteArray = Encoding.Unicode.GetBytes( 
				EncryptAndEncodeThis);
			//now return the byte array as a "safe for XMLDOM" Base64 String
			return Convert.ToBase64String(
				Encrypt(inputByteArray));

		}

		public byte[] Encrypt(byte[] EncryptThis)
		{
			//now encrypt the bytearray
			MemoryStream ms = new MemoryStream();
			DESCryptoServiceProvider DESProv = new
				DESCryptoServiceProvider();
			CryptoStream cs = new CryptoStream(ms
				, DESProv.CreateEncryptor(_key, _IV) 
				, CryptoStreamMode.Write);
			cs.Write(EncryptThis
				, 0 
				, EncryptThis.Length);
			cs.FlushFinalBlock();
			return ms.ToArray();
		}

		public string DecodeDecryptString(string EncryptedAndEncodedData)
		{

			byte[]inputByteArray = (byte[])Array.CreateInstance(typeof(byte),
				EncryptedAndEncodedData.Length);
			// we have a base 64 encoded string so first must decode 
			//to regular unencoded (encrypted) string
			inputByteArray = Convert.FromBase64String(EncryptedAndEncodedData);
			//now decrypt the regular string
			return Encoding.Unicode.GetString(Decrypt(inputByteArray));
		}

		public byte[] Decrypt(byte[] EncryptedData)
		{
			MemoryStream ms = new MemoryStream();
			DESCryptoServiceProvider DESProv = new
				DESCryptoServiceProvider();
			CryptoStream cs = new CryptoStream(ms
				, DESProv.CreateDecryptor(_key, _IV) 
				, CryptoStreamMode.Write);
			cs.Write(EncryptedData, 0, EncryptedData.Length);
			cs.FlushFinalBlock();
			return ms.ToArray();
		}

		public CryptoStream WrapStreamForEncryption(Stream stream)
		{
			DESCryptoServiceProvider DESProv = new
				DESCryptoServiceProvider();
			CryptoStream cs = new CryptoStream(stream
				, DESProv.CreateEncryptor(_key, _IV) 
				, CryptoStreamMode.Write);
			return cs;
		}

		public CryptoStream WrapStreamForDecryption(Stream stream)
		{
			DESCryptoServiceProvider DESProv = new
				DESCryptoServiceProvider();
			CryptoStream cs = new CryptoStream(stream
				, DESProv.CreateDecryptor(_key, _IV) 
				, CryptoStreamMode.Read);
			return cs;
		}

		private static MD5CryptoServiceProvider _md5 = new MD5CryptoServiceProvider();

		public string ComputeHash(string Value)
		{
			return ByteArrayToString(_md5.ComputeHash(
				new UnicodeEncoding().GetBytes(Value)));
		} 

		private string ByteArrayToString(byte[] arrInput)
		{
			int i;
			StringBuilder sOutput = new StringBuilder(arrInput.Length);
			for (i=0;i < arrInput.Length; i++)
			{
				sOutput.Append(arrInput[i].ToString("X2"));
			}
			return sOutput.ToString();
		}
	}
}
